package main

import (
	"crypto/sha256"
	"encoding/json"
	"fmt"
	"git.randomchars.net/freenitori/log"
	"github.com/gin-gonic/gin"
	"io"
	"io/ioutil"
	"net/http"
	"os"
	"time"
)

type File struct {
	Hash   	 string  `json:"Hash"`
	FileName string  `json:"FileName"`
	Date     int64  `json:"Date"`
}

func main() {
	r := gin.Default()
	r.LoadHTMLGlob("templates/*.html")
	r.GET("/", indexHandler)
	r.POST("/upload", uploadHandler)
	r.GET("/download/:hash", downloadHandler)
	r.GET("/read", readHandler)
	r.GET("/files/:hash", fileHandler)
	log.Info("The website is running at http://localhost:8080")
	r.Run()
}

func indexHandler(c *gin.Context){
	c.HTML(http.StatusOK,"index.html",gin.H{
		"title": "Hello World",
	})
}

func uploadHandler(c *gin.Context){
	file, fileHeader, err := c.Request.FormFile("file")
	if err != nil {
		c.String(http.StatusBadRequest, fmt.Sprintf("file err : %s", err.Error()))
		return
	}

	data, err := ioutil.ReadAll(file)

	hash := fmt.Sprintf("%x", sha256.Sum256(data))

	defer file.Close()

	err = os.MkdirAll("./files", os.ModePerm)
	if err != nil {
		c.String(http.StatusBadRequest, fmt.Sprintf("directory creating err : %s", err.Error()))
		return
	}

	err = os.MkdirAll(fmt.Sprintf("./files/%s", hash), os.ModePerm)
	if err != nil {
		c.String(http.StatusBadRequest, fmt.Sprintf("hash directory creating err : %s", err.Error()))
		return
	}

	dst, err := os.Create(fmt.Sprintf("./files/%s/file", hash))
	if err != nil {
		c.String(http.StatusBadRequest, fmt.Sprintf("file creating err : %s", err.Error()))
		return
	}

	FileHeader := make([]byte, 512)
	file.Read(FileHeader)

	fileData, _ := json.Marshal(struct {
		Hash string
		FileName string
		Date int64
	}{
		Hash: hash,
		FileName: fileHeader.Filename,
		Date: time.Now().UTC().Unix(),
	})

	jsonFile, _ := os.Create(fmt.Sprintf("./files/%s/info.json", hash))
	jsonFile.Write(fileData)
	defer dst.Close()

	_, err = io.Copy(dst, file)
	if err != nil {
		c.String(http.StatusBadRequest, fmt.Sprintf("file copy err : %s", err.Error()))
		return
	}

	c.JSON(http.StatusOK, gin.H{"data": fileData})
}

func downloadHandler(c *gin.Context){
	hash := c.Param("hash")
	var FileInfo File
	info, err := ioutil.ReadFile(fmt.Sprintf("./files/%s/info.json", hash))
	if err != nil {
		log.Fatal(err)
	}
	json.Unmarshal(info, &FileInfo)
	c.Header("Content-Description", "File Transfer")
	c.Header("Content-Transfer-Encoding", "binary")
	c.Header("Content-Disposition", "attachment; filename="+FileInfo.FileName)
	c.Header("Content-Type", "application/octet-stream")
	c.File(fmt.Sprintf("./files/%s/file", hash))
}

func readHandler(c *gin.Context){

	files, err := ioutil.ReadDir("./files")
	if err != nil {
		log.Fatal(err)
	}
	var FilesInfo File
	var data []File
	for _, f := range files {
		files, err := ioutil.ReadFile(fmt.Sprintf("./files/%s/info.json", f.Name()))
		if err != nil {
			log.Fatal(err)
		}
		json.Unmarshal(files, &FilesInfo)
		data = append(data,FilesInfo)
	}
	c.Writer.Header().Set("Content-Type", "application/json")
	c.Writer.WriteHeader(http.StatusCreated)
	json.NewEncoder(c.Writer).Encode(data)
}

func fileHandler(c *gin.Context){
	hash := c.Param("hash")
	var FileInfo File
	info, err := ioutil.ReadFile(fmt.Sprintf("./files/%s/info.json", hash))
	if err != nil {
		log.Fatal(err)
	}
	json.Unmarshal(info, &FileInfo)
	tm := time.Unix(FileInfo.Date, 0)
	c.HTML(http.StatusOK,"download.html",gin.H{
		"title": fmt.Sprintf("Download - %s", FileInfo.FileName),
		"filename": FileInfo.FileName,
		"date": tm,
		"hash": FileInfo.Hash,
	})
}